@@ -19,15 +19,6 @@
 #include "elisaapplication.h"
 
 #include "musiclistenersmanager.h"
-#include "models/allalbumsproxymodel.h"
-#include "models/alltracksproxymodel.h"
-#include "models/allartistsproxymodel.h"
-#include "models/singleartistproxymodel.h"
-#include "models/singlealbumproxymodel.h"
-
-#if defined KF5KIO_FOUND && KF5KIO_FOUND
-#include "models/filebrowserproxymodel.h"
-#endif
 
 #include "mediaplaylist.h"
 #include "audiowrapper.h"
@@ -65,6 +56,7 @@
 #include <QUrl>
 #include <QFileInfo>
 #include <QDir>
+#include <QKeyEvent>
 #include <QDebug>
 
 #include <memory>
@@ -75,8 +67,8 @@ public:
 
     explicit ElisaApplicationPrivate(QObject *parent)
 #if defined KF5XmlGui_FOUND && KF5XmlGui_FOUND
-          : mCollection(parent)
-#endif
+        : mCollection(parent)
+    #endif
     {
         Q_UNUSED(parent)
 
@@ -91,30 +83,10 @@ public:
     KActionCollection mCollection;
 #endif
 
-    QStringList mArguments;
+    ElisaUtils::EntryDataList mArguments;
 
     std::unique_ptr<MusicListenersManager> mMusicManager;
 
-    std::unique_ptr<AllAlbumsProxyModel> mAllAlbumsProxyModel;
-
-    std::unique_ptr<AllArtistsProxyModel> mAllArtistsProxyModel;
-
-    std::unique_ptr<AllTracksProxyModel> mAllTracksProxyModel;
-
-    std::unique_ptr<AllArtistsProxyModel> mAllGenresProxyModel;
-
-    std::unique_ptr<AllArtistsProxyModel> mAllComposersProxyModel;
-
-    std::unique_ptr<AllArtistsProxyModel> mAllLyricistsProxyModel;
-
-    std::unique_ptr<SingleArtistProxyModel> mSingleArtistProxyModel;
-
-    std::unique_ptr<SingleAlbumProxyModel> mSingleAlbumProxyModel;
-
-#if defined KF5KIO_FOUND && KF5KIO_FOUND
-    std::unique_ptr<FileBrowserProxyModel> mFileBrowserProxyModel;
-#endif
-
     std::unique_ptr<MediaPlayList> mMediaPlayList;
 
     std::unique_ptr<AudioWrapper> mAudioWrapper;
@@ -178,6 +150,21 @@ void ElisaApplication::setupActions(const QString &actionName)
         togglePlaylistAction->setText(QStringLiteral("Toggle Playlist"));
     }
 
+    if (actionName == QStringLiteral("Seek") && KAuthorized::authorizeAction(actionName)) {
+            auto seekAction = d->mCollection.addAction(actionName, this, &ElisaApplication::seek);
+            d->mCollection.setDefaultShortcut(seekAction, QKeySequence(tr("Shift+Right")));
+    }
+
+    if (actionName == QStringLiteral("Scrub") && KAuthorized::authorizeAction(actionName)) {
+            auto scrubAction = d->mCollection.addAction(actionName, this, &ElisaApplication::scrub);
+            d->mCollection.setDefaultShortcut(scrubAction, QKeySequence(tr("Shift+Left")));
+    }
+
+    if (actionName == QStringLiteral("Play-Pause") && KAuthorized::authorizeAction(actionName)) {
+            auto playPauseAction = d->mCollection.addAction(actionName, this, &ElisaApplication::playPause);
+            d->mCollection.setDefaultShortcut(playPauseAction, QKeySequence(tr("Space")));
+    }
+
     if (actionName == QStringLiteral("edit_find") && KAuthorized::authorizeAction(actionName)) {
         auto findAction = KStandardAction::find(this, &ElisaApplication::find, this);
         d->mCollection.addAction(findAction->objectName(), findAction);
@@ -187,7 +174,7 @@ void ElisaApplication::setupActions(const QString &actionName)
 #endif
 }
 
-void ElisaApplication::setArguments(const QStringList &newArguments)
+void ElisaApplication::setArguments(const ElisaUtils::EntryDataList &newArguments)
 {
     if (d->mArguments == newArguments) {
         return;
@@ -197,7 +184,9 @@ void ElisaApplication::setArguments(const QStringList &newArguments)
     Q_EMIT argumentsChanged();
 
     if (!d->mArguments.isEmpty()) {
-        Q_EMIT enqueue(d->mArguments);
+        Q_EMIT enqueue(d->mArguments, ElisaUtils::FileName,
+                       ElisaUtils::PlayListEnqueueMode::AppendPlayList,
+                       ElisaUtils::PlayListEnqueueTriggerPlay::TriggerPlay);
     }
 }
 
@@ -209,10 +198,23 @@ void ElisaApplication::activateActionRequested(const QString &actionName, const
 
 void ElisaApplication::activateRequested(const QStringList &arguments, const QString &workingDirectory)
 {
-    auto realArguments = arguments;
-    if (realArguments.size() > 1) {
-        realArguments.removeFirst();
-        Q_EMIT enqueue(checkFileListAndMakeAbsolute(realArguments, workingDirectory));
+    if (arguments.size() > 1) {
+        auto realArguments = ElisaUtils::EntryDataList{};
+
+        bool isFirst = true;
+        for (const auto &oneArgument : arguments) {
+            if (isFirst) {
+                isFirst = false;
+                continue;
+            }
+
+            realArguments.push_back(ElisaUtils::EntryData{0, oneArgument});
+        }
+
+        Q_EMIT enqueue(checkFileListAndMakeAbsolute(realArguments, workingDirectory),
+                       ElisaUtils::FileName,
+                       ElisaUtils::PlayListEnqueueMode::AppendPlayList,
+                       ElisaUtils::PlayListEnqueueTriggerPlay::TriggerPlay);
     }
 }
 
@@ -277,19 +279,26 @@ void ElisaApplication::find() {}
 
 void ElisaApplication::togglePlaylist() {}
 
-QStringList ElisaApplication::checkFileListAndMakeAbsolute(const QStringList &filesList, const QString &workingDirectory) const
+void ElisaApplication::seek() {}
+
+void ElisaApplication::scrub() {}
+
+void ElisaApplication::playPause() {}
+
+ElisaUtils::EntryDataList ElisaApplication::checkFileListAndMakeAbsolute(const ElisaUtils::EntryDataList &filesList,
+                                                                            const QString &workingDirectory) const
 {
-    QStringList filesToOpen;
+    auto filesToOpen = ElisaUtils::EntryDataList{};
 
     for (const auto &oneFile : filesList) {
-        auto newFile = QFileInfo(oneFile);
+        auto newFile = QFileInfo(std::get<1>(oneFile));
 
         if (newFile.isRelative()) {
-            newFile = QFileInfo(workingDirectory + QStringLiteral("/") + oneFile);
+            newFile = QFileInfo(workingDirectory + QStringLiteral("/") + std::get<1>(oneFile));
         }
 
         if (newFile.exists()) {
-            filesToOpen.push_back(newFile.canonicalFilePath());
+            filesToOpen.push_back(ElisaUtils::EntryData{0, newFile.canonicalFilePath()});
         }
     }
 
@@ -308,27 +317,6 @@ void ElisaApplication::initializeModels()
 {
     d->mMusicManager = std::make_unique<MusicListenersManager>();
     Q_EMIT musicManagerChanged();
-    d->mAllAlbumsProxyModel = std::make_unique<AllAlbumsProxyModel>();
-    Q_EMIT allAlbumsProxyModelChanged();
-    d->mAllArtistsProxyModel = std::make_unique<AllArtistsProxyModel>();
-    Q_EMIT allArtistsProxyModelChanged();
-    d->mAllGenresProxyModel = std::make_unique<AllArtistsProxyModel>();
-    Q_EMIT allGenresProxyModelChanged();
-    d->mAllComposersProxyModel = std::make_unique<AllArtistsProxyModel>();
-    Q_EMIT allComposersProxyModelChanged();
-    d->mAllLyricistsProxyModel = std::make_unique<AllArtistsProxyModel>();
-    Q_EMIT allLyricistsProxyModelChanged();
-    d->mAllTracksProxyModel = std::make_unique<AllTracksProxyModel>();
-    Q_EMIT allTracksProxyModelChanged();
-    d->mSingleArtistProxyModel = std::make_unique<SingleArtistProxyModel>();
-    Q_EMIT singleArtistProxyModelChanged();
-    d->mSingleAlbumProxyModel = std::make_unique<SingleAlbumProxyModel>();
-    Q_EMIT singleAlbumProxyModelChanged();
-
-#if defined KF5KIO_FOUND && KF5KIO_FOUND
-    d->mFileBrowserProxyModel = std::make_unique<FileBrowserProxyModel>();
-    Q_EMIT fileBrowserProxyModelChanged();
-#endif
 
     d->mMediaPlayList = std::make_unique<MediaPlayList>();
     Q_EMIT mediaPlayListChanged();
@@ -336,46 +324,11 @@ void ElisaApplication::initializeModels()
     d->mMusicManager->setElisaApplication(this);
 
     d->mMediaPlayList->setMusicListenersManager(d->mMusicManager.get());
-    QObject::connect(this, &ElisaApplication::enqueue, d->mMediaPlayList.get(), &MediaPlayList::enqueueAndPlay);
-
-    d->mAllAlbumsProxyModel->setSourceModel(d->mMusicManager->allAlbumsModel());
-    d->mAllArtistsProxyModel->setSourceModel(d->mMusicManager->allArtistsModel());
-    d->mAllGenresProxyModel->setSourceModel(d->mMusicManager->allGenresModel());
-    d->mAllComposersProxyModel->setSourceModel(d->mMusicManager->allComposersModel());
-    d->mAllLyricistsProxyModel->setSourceModel(d->mMusicManager->allLyricistsModel());
-    d->mAllTracksProxyModel->setSourceModel(d->mMusicManager->allTracksModel());
-    d->mSingleArtistProxyModel->setSourceModel(d->mMusicManager->allAlbumsModel());
-    d->mSingleAlbumProxyModel->setSourceModel(d->mMusicManager->albumModel());
-
-    QObject::connect(d->mAllAlbumsProxyModel.get(), &AllAlbumsProxyModel::albumToEnqueue,
-                     d->mMediaPlayList.get(), static_cast<void (MediaPlayList::*)(const QList<MusicAlbum> &,
-                                                                         ElisaUtils::PlayListEnqueueMode,
-                                                                         ElisaUtils::PlayListEnqueueTriggerPlay)>(&MediaPlayList::enqueue));
-
-    QObject::connect(d->mAllArtistsProxyModel.get(), &AllArtistsProxyModel::artistToEnqueue,
-                     d->mMediaPlayList.get(), &MediaPlayList::enqueueArtists);
-
-    QObject::connect(d->mAllTracksProxyModel.get(), &AllTracksProxyModel::trackToEnqueue,
-                     d->mMediaPlayList.get(), static_cast<void (MediaPlayList::*)(const QList<MusicAudioTrack> &,
-                                                                         ElisaUtils::PlayListEnqueueMode,
-                                                                         ElisaUtils::PlayListEnqueueTriggerPlay)>(&MediaPlayList::enqueue));
-
-    QObject::connect(d->mSingleArtistProxyModel.get(), &SingleArtistProxyModel::albumToEnqueue,
-                     d->mMediaPlayList.get(), static_cast<void (MediaPlayList::*)(const QList<MusicAlbum> &,
-                                                                         ElisaUtils::PlayListEnqueueMode,
-                                                                         ElisaUtils::PlayListEnqueueTriggerPlay)>(&MediaPlayList::enqueue));
-
-    QObject::connect(d->mSingleAlbumProxyModel.get(), &SingleAlbumProxyModel::trackToEnqueue,
-                     d->mMediaPlayList.get(), static_cast<void (MediaPlayList::*)(const QList<MusicAudioTrack> &,
-                                                                         ElisaUtils::PlayListEnqueueMode,
-                                                                         ElisaUtils::PlayListEnqueueTriggerPlay)>(&MediaPlayList::enqueue));
-
-#if defined KF5KIO_FOUND && KF5KIO_FOUND
-    QObject::connect(d->mFileBrowserProxyModel.get(), &FileBrowserProxyModel::filesToEnqueue,
-                     d->mMediaPlayList.get(), static_cast<void (MediaPlayList::*)(const QList<QUrl> &,
-                                                                         ElisaUtils::PlayListEnqueueMode,
-                                                                         ElisaUtils::PlayListEnqueueTriggerPlay)>(&MediaPlayList::enqueue));
-#endif
+    QObject::connect(this, &ElisaApplication::enqueue,
+                     d->mMediaPlayList.get(), static_cast<void (MediaPlayList::*)(const ElisaUtils::EntryDataList&,
+                                                                                  ElisaUtils::PlayListEntryType,
+                                                                                  ElisaUtils::PlayListEnqueueMode,
+                                                                                  ElisaUtils::PlayListEnqueueTriggerPlay)>(&MediaPlayList::enqueue));
 }
 
 void ElisaApplication::initializePlayer()
@@ -400,19 +353,24 @@ void ElisaApplication::initializePlayer()
     QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::playerPause, d->mAudioWrapper.get(), &AudioWrapper::pause);
     QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::playerStop, d->mAudioWrapper.get(), &AudioWrapper::stop);
     QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::seek, d->mAudioWrapper.get(), &AudioWrapper::seek);
+    QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::saveUndoPositionInAudioWrapper, d->mAudioWrapper.get(), &AudioWrapper::saveUndoPosition);
+    QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::restoreUndoPositionInAudioWrapper, d->mAudioWrapper.get(), &AudioWrapper::restoreUndoPosition);
     QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::skipNextTrack, d->mMediaPlayList.get(), &MediaPlayList::skipNextTrack);
     QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::sourceInError, d->mMediaPlayList.get(), &MediaPlayList::trackInError);
     QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::sourceInError, d->mMusicManager.get(), &MusicListenersManager::playBackError);
     QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::playerSourceChanged, d->mAudioWrapper.get(), &AudioWrapper::setSource);
+    QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::startedPlayingTrack,
+                     d->mMusicManager->viewDatabase(), &DatabaseInterface::trackHasStartedPlaying);
 
     QObject::connect(d->mMediaPlayList.get(), &MediaPlayList::ensurePlay, d->mAudioControl.get(), &ManageAudioPlayer::ensurePlay);
     QObject::connect(d->mMediaPlayList.get(), &MediaPlayList::playListFinished, d->mAudioControl.get(), &ManageAudioPlayer::playListFinished);
     QObject::connect(d->mMediaPlayList.get(), &MediaPlayList::currentTrackChanged, d->mAudioControl.get(), &ManageAudioPlayer::setCurrentTrack);
+    QObject::connect(d->mMediaPlayList.get(), &MediaPlayList::clearPlayListPlayer, d->mAudioControl.get(), &ManageAudioPlayer::saveForUndoClearPlaylist);
+    QObject::connect(d->mMediaPlayList.get(), &MediaPlayList::undoClearPlayListPlayer, d->mAudioControl.get(), &ManageAudioPlayer::restoreForUndoClearPlaylist);
 
-    QObject::connect(d->mAudioWrapper.get(), &AudioWrapper::playbackStateChanged, d->mAudioControl.get(),
-                     [this](QMediaPlayer::State state) {
-        d->mAudioControl->setPlayerPlaybackState(static_cast<int>(state));
-    });
+
+    QObject::connect(d->mAudioWrapper.get(), &AudioWrapper::playbackStateChanged,
+                     d->mAudioControl.get(), &ManageAudioPlayer::setPlayerPlaybackState);
     QObject::connect(d->mAudioWrapper.get(), &AudioWrapper::statusChanged, d->mAudioControl.get(), &ManageAudioPlayer::setPlayerStatus);
     QObject::connect(d->mAudioWrapper.get(), &AudioWrapper::errorChanged, d->mAudioControl.get(), &ManageAudioPlayer::setPlayerError);
     QObject::connect(d->mAudioWrapper.get(), &AudioWrapper::durationChanged, d->mAudioControl.get(), &ManageAudioPlayer::setAudioDuration);
@@ -427,15 +385,20 @@ void ElisaApplication::initializePlayer()
 
     d->mManageHeaderBar->setTitleRole(MediaPlayList::TitleRole);
     d->mManageHeaderBar->setAlbumRole(MediaPlayList::AlbumRole);
+    d->mManageHeaderBar->setAlbumArtistRole(MediaPlayList::AlbumArtistRole);
     d->mManageHeaderBar->setArtistRole(MediaPlayList::ArtistRole);
-    d->mManageHeaderBar->setImageRole(MediaPlayList::ImageRole);
+    d->mManageHeaderBar->setFileNameRole(MediaPlayList::ResourceRole);
+    d->mManageHeaderBar->setImageRole(MediaPlayList::ImageUrlRole);
+    d->mManageHeaderBar->setDatabaseIdRole(MediaPlayList::DatabaseIdRole);
     d->mManageHeaderBar->setAlbumIdRole(MediaPlayList::AlbumIdRole);
     d->mManageHeaderBar->setIsValidRole(MediaPlayList::IsValidRole);
     d->mManageHeaderBar->setPlayListModel(d->mMediaPlayList.get());
     QObject::connect(d->mMediaPlayList.get(), &MediaPlayList::currentTrackChanged, d->mManageHeaderBar.get(), &ManageHeaderBar::setCurrentTrack);
 
     if (!d->mArguments.isEmpty()) {
-        Q_EMIT enqueue(d->mArguments);
+        Q_EMIT enqueue(d->mArguments, ElisaUtils::FileName,
+                       ElisaUtils::PlayListEnqueueMode::AppendPlayList,
+                       ElisaUtils::PlayListEnqueueTriggerPlay::TriggerPlay);
     }
 }
 
@@ -462,7 +425,29 @@ QString ElisaApplication::iconName(const QIcon& icon)
     return icon.name();
 }
 
-const QStringList &ElisaApplication::arguments() const
+void ElisaApplication::installKeyEventFilter(QObject *object)
+{
+    if(!object) {
+        return;
+    }
+
+    object->installEventFilter(this);
+}
+
+bool ElisaApplication::eventFilter(QObject *object, QEvent *event)
+{
+    Q_UNUSED(object);
+
+    QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
+    auto playPauseAction = d->mCollection.action(tr("Play-Pause"));
+    if (keyEvent->key() == Qt::Key_Space && playPauseAction->shortcut()[0] == Qt::Key_Space) {
+        return true;
+    }
+
+    return false;
+}
+
+const ElisaUtils::EntryDataList &ElisaApplication::arguments() const
 {
     return d->mArguments;
 }
@@ -472,55 +457,6 @@ MusicListenersManager *ElisaApplication::musicManager() const
     return d->mMusicManager.get();
 }
 
-QSortFilterProxyModel *ElisaApplication::allAlbumsProxyModel() const
-{
-    return d->mAllAlbumsProxyModel.get();
-}
-
-QSortFilterProxyModel *ElisaApplication::allArtistsProxyModel() const
-{
-    return d->mAllArtistsProxyModel.get();
-}
-
-QSortFilterProxyModel *ElisaApplication::allGenresProxyModel() const
-{
-    return d->mAllGenresProxyModel.get();
-}
-
-QSortFilterProxyModel *ElisaApplication::allComposersProxyModel() const
-{
-    return d->mAllComposersProxyModel.get();
-}
-
-QSortFilterProxyModel *ElisaApplication::allLyricistsProxyModel() const
-{
-    return d->mAllLyricistsProxyModel.get();
-}
-
-QSortFilterProxyModel *ElisaApplication::allTracksProxyModel() const
-{
-    return d->mAllTracksProxyModel.get();
-}
-
-QSortFilterProxyModel *ElisaApplication::singleArtistProxyModel() const
-{
-    return d->mSingleArtistProxyModel.get();
-}
-
-QSortFilterProxyModel *ElisaApplication::singleAlbumProxyModel() const
-{
-    return d->mSingleAlbumProxyModel.get();
-}
-
-QSortFilterProxyModel *ElisaApplication::fileBrowserProxyModel() const
-{
-#if defined KF5KIO_FOUND && KF5KIO_FOUND
-    return d->mFileBrowserProxyModel.get();
-#else
-    return nullptr;
-#endif
-}
-
 MediaPlayList *ElisaApplication::mediaPlayList() const
 {
     return d->mMediaPlayList.get();
