@@ -33,27 +33,12 @@ class ELISALIB_EXPORT LocalFileListing : public AbstractFileListing
 
     Q_OBJECT
 
-    Q_PROPERTY(QString rootPath
-               READ rootPath
-               WRITE setRootPath
-               NOTIFY rootPathChanged)
-
 public:
 
     explicit LocalFileListing(QObject *parent = nullptr);
 
     ~LocalFileListing() override;
 
-    QString rootPath() const;
-
-Q_SIGNALS:
-
-    void rootPathChanged();
-
-public Q_SLOTS:
-
-    void setRootPath(const QString &rootPath);
-
 private:
 
     void executeInit(QHash<QUrl, QDateTime> allFiles) override;
