@@ -21,17 +21,17 @@
 #include "elisaLib_export.h"
 
 #include "musicaudiotrack.h"
-#include "musicalbum.h"
-#include "musicartist.h"
 #include "elisautils.h"
+#include "databaseinterface.h"
 
 #include <QAbstractListModel>
 #include <QVector>
-#include <utility>
 #include <QMediaPlayer>
 
+#include <utility>
+#include <tuple>
+
 class MediaPlayListPrivate;
-class DatabaseInterface;
 class MusicListenersManager;
 class MediaPlayListEntry;
 class QDebug;
@@ -75,14 +75,19 @@ class ELISALIB_EXPORT MediaPlayList : public QAbstractListModel
 public:
 
     enum ColumnsRoles {
-        IsValidRole = Qt::UserRole + 1,
-        TitleRole,
+        TitleRole = DatabaseInterface::TitleRole,
+        SecondaryTextRole,
+        ImageUrlRole,
+        ShadowForImageRole,
+        ChildModelRole,
         DurationRole,
+        StringDurationRole,
         MilliSecondsDurationRole,
-        TrackDataRole,
         ArtistRole,
-        AlbumArtistRole,
+        AllArtistsRole,
+        HighestTrackRating,
         AlbumRole,
+        AlbumArtistRole,
         TrackNumberRole,
         DiscNumberRole,
         RatingRole,
@@ -94,16 +99,25 @@ public:
         ChannelsRole,
         BitRateRole,
         SampleRateRole,
-        ImageRole,
-        AlbumIdRole,
         ResourceRole,
+        IdRole,
+        DatabaseIdRole,
+        IsSingleDiscAlbumRole,
+        ContainerDataRole,
+        IsPartialDataRole,
+        AlbumIdRole,
+        HasEmbeddedCover,
+        FileModificationTime,
+        FirstPlayDate,
+        LastPlayDate,
+        PlayCounter,
+        PlayFrequency,
+        ElementTypeRole,
+        IsValidRole,
+        TrackDataRole,
         CountRole,
         IsPlayingRole,
-        HasAlbumHeader,
-        IsSingleDiscAlbumHeader,
-        SecondaryTextRole,
-        ImageUrlRole,
-        ShadowForImageRole,
+        AlbumSectionRole,
     };
 
     Q_ENUM(ColumnsRoles)
@@ -116,6 +130,10 @@ public:
 
     Q_ENUM(PlayState)
 
+    using ListTrackDataType = DatabaseInterface::ListTrackDataType;
+
+    using TrackDataType = DatabaseInterface::TrackDataType;
+
     explicit MediaPlayList(QObject *parent = nullptr);
 
     ~MediaPlayList() override;
@@ -153,16 +171,15 @@ public:
     bool repeatPlay() const;
 
 Q_SIGNALS:
+    void displayUndoInline();
+
+    void hideUndoInline();
 
     void newTrackByNameInList(const QString &title, const QString &artist, const QString &album, int trackNumber, int discNumber);
 
-    void newTrackByFileNameInList(const QUrl &fileName);
-
-    void newTrackByIdInList(qulonglong newTrackId);
-
-    void newArtistInList(const QString &artist);
-
-    void trackHasBeenAdded(const QString &title, const QUrl &image);
+    void newEntryInList(qulonglong newDatabaseId,
+                        const QString &entryTitle,
+                        ElisaUtils::PlayListEntryType databaseIdType);
 
     void persistentStateChanged();
 
@@ -172,6 +189,10 @@ Q_SIGNALS:
 
     void currentTrackChanged(QPersistentModelIndex currentTrack);
 
+    void clearPlayListPlayer();
+
+    void undoClearPlayListPlayer();
+
     void currentTrackRowChanged();
 
     void randomPlayChanged();
@@ -192,9 +213,12 @@ public Q_SLOTS:
 
     void removeSelection(QList<int> selection);
 
-    void albumAdded(const QList<MusicAudioTrack> &tracks);
+    void tracksListAdded(qulonglong newDatabaseId,
+                         const QString &entryTitle,
+                         ElisaUtils::PlayListEntryType databaseIdType,
+                         const MediaPlayList::ListTrackDataType &tracks);
 
-    void trackChanged(const MusicAudioTrack &track);
+    void trackChanged(const MediaPlayList::TrackDataType &track);
 
     void trackRemoved(qulonglong trackId);
 
@@ -214,55 +238,32 @@ public Q_SLOTS:
 
     void loadPlaylist(const QUrl &fileName);
 
-    void enqueue(qulonglong newTrackId);
+    void enqueue(const ElisaUtils::EntryData &newEntry, ElisaUtils::PlayListEntryType databaseIdType);
 
-    void enqueue(const MusicAudioTrack &newTrack);
+    void enqueue(const ElisaUtils::EntryDataList &newEntries, ElisaUtils::PlayListEntryType databaseIdType);
 
-    void enqueue(const MediaPlayListEntry &newEntry, const MusicAudioTrack &audioTrack = {});
-
-    void enqueue(const MusicAlbum &album);
-
-    void enqueue(const MusicArtist &artist);
-
-    void enqueue(const QString &artistName);
-
-    void enqueue(const QUrl &fileName);
-
-    void enqueue(const QStringList &files);
-
-    void enqueue(const QList<qulonglong> &newTrackId);
-
-    void enqueue(const QList<MusicAlbum> &albums,
+    void enqueue(qulonglong newEntryDatabaseId, const QString &newEntryTitle,
+                 ElisaUtils::PlayListEntryType databaseIdType,
                  ElisaUtils::PlayListEnqueueMode enqueueMode,
                  ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay);
 
-    void enqueue(const QList<MusicAudioTrack> &tracks,
+    void enqueue(const ElisaUtils::EntryData &newEntry,
+                 ElisaUtils::PlayListEntryType databaseIdType,
                  ElisaUtils::PlayListEnqueueMode enqueueMode,
                  ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay);
 
-    void enqueueArtists(const QList<QString> &artistName,
-                        ElisaUtils::PlayListEnqueueMode enqueueMode,
-                        ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay);
-
-    void enqueue(const QList<QUrl> &tracks,
+    void enqueue(const ElisaUtils::EntryDataList &newEntries,
+                 ElisaUtils::PlayListEntryType databaseIdType,
                  ElisaUtils::PlayListEnqueueMode enqueueMode,
                  ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay);
 
-    void enqueueAndPlay(const QStringList &files);
+    void replaceAndPlay(const ElisaUtils::EntryData &newEntry, ElisaUtils::PlayListEntryType databaseIdType);
 
-    void replaceAndPlay(qulonglong newTrackId);
+    void enqueueRestoredEntry(const MediaPlayListEntry &newEntry);
 
-    void replaceAndPlay(const MusicAudioTrack &newTrack);
+    void trackInError(const QUrl &sourceInError, QMediaPlayer::Error playerError);
 
-    void replaceAndPlay(const MusicAlbum &album);
-
-    void replaceAndPlay(const MusicArtist &artist);
-
-    void replaceAndPlay(const QString &artistName);
-
-    void replaceAndPlay(const QUrl &fileName);
-
-    void trackInError(QUrl sourceInError, QMediaPlayer::Error playerError);
+    void undoClearPlayList();
 
 private Q_SLOTS:
 
@@ -271,8 +272,9 @@ private Q_SLOTS:
     void loadPlayListLoadFailed();
 
 private:
+    void displayOrHideUndoInline(bool value);
 
-    bool rowHasHeader(int row) const;
+    void clearPlayList(bool prepareUndo);
 
     void resetCurrentTrack();
 
@@ -284,8 +286,23 @@ private:
 
     void restoreRepeatPlay();
 
+    void enqueueArtist(const QString &artistName);
+
+    void enqueueFilesList(const ElisaUtils::EntryDataList &newEntries);
+
+    void enqueueTracksListById(const ElisaUtils::EntryDataList &newEntries);
+
+    void enqueueOneEntry(const ElisaUtils::EntryData &entryData, ElisaUtils::PlayListEntryType type);
+
+    void enqueueMultipleEntries(const ElisaUtils::EntryDataList &entriesData, ElisaUtils::PlayListEntryType type);
+
+    void enqueueCommon();
+
+    void copyD();
+
     std::unique_ptr<MediaPlayListPrivate> d;
 
+    std::unique_ptr<MediaPlayListPrivate> dOld;
 };
 
 class MediaPlayListEntry
@@ -297,9 +314,9 @@ public:
     explicit MediaPlayListEntry(qulonglong id) : mId(id), mIsValid(true) {
     }
 
-    MediaPlayListEntry(QString title, QString artist, QString album, int trackNumber, int discNumber)
+    MediaPlayListEntry(QString title, QString artist, QString album, int trackNumber, int discNumber, ElisaUtils::PlayListEntryType entryType = ElisaUtils::Unknown)
         : mTitle(std::move(title)), mAlbum(std::move(album)), mArtist(std::move(artist)),
-          mTrackNumber(trackNumber), mDiscNumber(discNumber) {
+          mTrackNumber(trackNumber), mDiscNumber(discNumber), mEntryType(entryType) {
     }
 
     explicit MediaPlayListEntry(const MusicAudioTrack &track)
@@ -307,35 +324,50 @@ public:
           mDiscNumber(track.discNumber()), mId(track.databaseId()), mIsValid(true) {
     }
 
-    explicit MediaPlayListEntry(QString artist) : mArtist(std::move(artist)), mIsArtist(true) {
+    explicit MediaPlayListEntry(const MediaPlayList::TrackDataType &track)
+        : mTitle(track[DatabaseInterface::TitleRole]),
+          mAlbum(track[DatabaseInterface::AlbumRole]),
+          mTrackNumber(track[DatabaseInterface::TrackNumberRole]),
+          mDiscNumber(track[DatabaseInterface::DiscNumberRole]),
+          mId(track[DatabaseInterface::DatabaseIdRole].toULongLong()),
+          mIsValid(true) {
+    }
+
+    explicit MediaPlayListEntry(QString artist) : mArtist(std::move(artist)), mEntryType(ElisaUtils::Artist) {
     }
 
     explicit MediaPlayListEntry(QUrl fileName) : mTrackUrl(std::move(fileName)) {
     }
 
-    QString mTitle;
+    explicit MediaPlayListEntry(qulonglong id,
+                                const QString &entryTitle,
+                                ElisaUtils::PlayListEntryType type)
+        : mTitle(entryTitle), mId(id), mIsValid(type == ElisaUtils::Track), mEntryType(type) {
+    }
 
-    QString mAlbum;
+    QVariant mTitle;
 
-    QString mArtist;
+    QVariant mAlbum;
 
-    QUrl mTrackUrl;
+    QVariant mArtist;
 
-    int mTrackNumber = -1;
+    QVariant mTrackUrl;
 
-    int mDiscNumber = -1;
+    QVariant mTrackNumber = -1;
+
+    QVariant mDiscNumber = -1;
 
     qulonglong mId = 0;
 
     bool mIsValid = false;
 
-    bool mIsArtist = false;
+    ElisaUtils::PlayListEntryType mEntryType = ElisaUtils::PlayListEntryType::Unknown;
 
     MediaPlayList::PlayState mIsPlaying = MediaPlayList::NotPlaying;
 
 };
 
-QDebug operator<<(QDebug stream, const MediaPlayListEntry &data);
+QDebug operator<<(const QDebug &stream, const MediaPlayListEntry &data);
 
 
 
