@@ -37,9 +37,10 @@ public:
 
 Q_SIGNALS:
 
-    void trackToEnqueue(QList<MusicAudioTrack> newTracks,
-                        ElisaUtils::PlayListEnqueueMode enqueueMode,
-                        ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay);
+    void entriesToEnqueue(const ElisaUtils::EntryDataList &newEntries,
+                          ElisaUtils::PlayListEntryType databaseIdType,
+                          ElisaUtils::PlayListEnqueueMode enqueueMode,
+                          ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay);
 
 public Q_SLOTS:
 
@@ -51,6 +52,11 @@ protected:
 
     bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const override;
 
+private:
+
+    void genericEnqueueToPlayList(ElisaUtils::PlayListEnqueueMode enqueueMode,
+                                  ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay);
+
 };
 
 #endif // ALLTRACKSPROXYMODEL_H
