@@ -21,7 +21,6 @@
 
 #include "elisaLib_export.h"
 
-#include "filebrowsermodel.h"
 #include "musicaudiotrack.h"
 #include "filescanner.h"
 #include "elisautils.h"
@@ -70,6 +69,8 @@ public:
 
     bool sortedAscending() const;
 
+    void setSourceModel(QAbstractItemModel *sourceModel) override;
+
 public Q_SLOTS:
 
     void enqueueToPlayList();
@@ -84,9 +85,12 @@ public Q_SLOTS:
 
     void sortModel(Qt::SortOrder order);
 
+    void replaceAndPlayOfUrl(const QUrl &fileUrl);
+
 Q_SIGNALS:
 
-    void filesToEnqueue(QList<QUrl> newFiles,
+    void filesToEnqueue(const ElisaUtils::EntryDataList &newFiles,
+                        ElisaUtils::PlayListEntryType databaseIdType,
                         ElisaUtils::PlayListEnqueueMode enqueueMode,
                         ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay);
 
@@ -98,8 +102,9 @@ Q_SIGNALS:
 
     void sortedAscendingChanged();
 
-protected:
+    void loadPlayListFromUrl(const QUrl &playListUrl);
 
+protected:
 
     bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const override;
 
@@ -121,8 +126,6 @@ private:
 
     QThreadPool mThreadPool;
 
-    std::unique_ptr<FileBrowserModel> mFileModel;
-
 };
 
 #endif // FILEBROWSERPROXYMODEL_H
