@@ -20,6 +20,8 @@
 
 #include "elisaLib_export.h"
 
+#include "databaseinterface.h"
+
 #include <QString>
 #include <QTime>
 #include <QUrl>
@@ -36,12 +38,14 @@ class ELISALIB_EXPORT MusicAudioTrack
 
 public:
 
+    using TrackDataType = DatabaseInterface::TrackDataType;
+
     MusicAudioTrack();
 
     MusicAudioTrack(bool aValid, QString aId, QString aParentId, QString aTitle, QString aArtist, QString aAlbumName,
                     QString aAlbumArtist, int aTrackNumber, int aDiscNumber, QTime aDuration, QUrl aResourceURI,
                     const QDateTime &fileModificationTime, QUrl aAlbumCover, int rating, bool aIsSingleDiscAlbum,
-                    QString aGenre, QString aComposer, QString aLyricist);
+                    QString aGenre, QString aComposer, QString aLyricist, bool aHasEmbeddedCover);
 
     MusicAudioTrack(MusicAudioTrack &&other) noexcept;
 
@@ -59,6 +63,8 @@ public:
 
     bool operator !=(const MusicAudioTrack &other) const;
 
+    static MusicAudioTrack trackFromData(const DatabaseInterface::TrackDataType &data);
+
     void setValid(bool value);
 
     bool isValid() const;
@@ -107,6 +113,10 @@ public:
 
     QString lyricist() const;
 
+    void setLyrics(const QString &value);
+
+    QString lyrics() const;
+
     void setComment(const QString &value);
 
     QString comment() const;
@@ -121,10 +131,14 @@ public:
 
     int trackNumber() const;
 
+    bool trackNumberIsValid() const;
+
     void setDiscNumber(int value);
 
     int discNumber() const;
 
+    bool discNumberIsValid() const;
+
     void setYear(int value);
 
     int year() const;
@@ -133,14 +147,20 @@ public:
 
     int channels() const;
 
+    bool channelsIsValid() const;
+
     void setBitRate(int value);
 
     int bitRate() const;
 
+    bool bitRateIsValid() const;
+
     void setSampleRate(int value);
 
     int sampleRate() const;
 
+    bool sampleRateIsValid() const;
+
     void setDuration(QTime value);
 
     QTime duration() const;
@@ -161,6 +181,12 @@ public:
 
     bool isSingleDiscAlbum() const;
 
+    void setHasEmbeddedCover(bool value);
+
+    bool hasEmbeddedCover() const;
+
+    TrackDataType toTrackData() const;
+
 private:
 
     QSharedDataPointer<MusicAudioTrackPrivate> d;
