@@ -54,7 +54,7 @@ TracksListener::TracksListener(DatabaseInterface *database, QObject *parent) : Q
 TracksListener::~TracksListener()
 = default;
 
-void TracksListener::tracksAdded(const QList<MusicAudioTrack> &allTracks)
+void TracksListener::tracksAdded(const ListTrackDataType &allTracks)
 {
     for (const auto &oneTrack : allTracks) {
         if (d->mTracksByIdSet.contains(oneTrack.databaseId())) {
@@ -76,7 +76,7 @@ void TracksListener::tracksAdded(const QList<MusicAudioTrack> &allTracks)
                 continue;
             }
 
-            if (std::get<2>(*itTrack) != oneTrack.albumName()) {
+            if (std::get<2>(*itTrack) != oneTrack.album()) {
                 ++itTrack;
                 continue;
             }
@@ -91,7 +91,7 @@ void TracksListener::tracksAdded(const QList<MusicAudioTrack> &allTracks)
                 continue;
             }
 
-            Q_EMIT trackHasChanged(oneTrack);
+            Q_EMIT trackHasChanged(TrackDataType(oneTrack));
 
             d->mTracksByIdSet.insert(oneTrack.databaseId());
             itTrack = d->mTracksByNameSet.erase(itTrack);
@@ -106,7 +106,7 @@ void TracksListener::trackRemoved(qulonglong id)
     }
 }
 
-void TracksListener::trackModified(const MusicAudioTrack &modifiedTrack)
+void TracksListener::trackModified(const TrackDataType &modifiedTrack)
 {
     if (d->mTracksByIdSet.contains(modifiedTrack.databaseId())) {
         Q_EMIT trackHasChanged(modifiedTrack);
@@ -125,26 +125,40 @@ void TracksListener::trackByNameInList(const QString &title, const QString &arti
 
     d->mTracksByIdSet.insert(newTrackId);
 
-    auto newTrack = d->mDatabase->trackFromDatabaseId(newTrackId);
+    auto newTrack = d->mDatabase->trackDataFromDatabaseId(newTrackId);
 
-    if (newTrack.isValid()) {
+    if (!newTrack.isEmpty()) {
         Q_EMIT trackHasChanged(newTrack);
     }
 }
 
 void TracksListener::trackByFileNameInList(const QUrl &fileName)
 {
-    qDebug() << "TracksListener::trackByFileNameInList" << fileName;
     auto newTrackId = d->mDatabase->trackIdFromFileName(fileName);
-    qDebug() << "TracksListener::trackByFileNameInList" << fileName << newTrackId;
     if (newTrackId == 0) {
         auto newTrack = scanOneFile(fileName);
 
-        qDebug() << "TracksListener::trackByFileNameInList" << fileName << newTrack;
-
         if (newTrack.isValid()) {
-            qDebug() << "TracksListener::trackByFileNameInList" << "trackHasChanged" << newTrack;
-            Q_EMIT trackHasChanged(newTrack);
+            auto oneData = DatabaseInterface::TrackDataType{};
+
+            if (!newTrack.title().isEmpty()) {
+                oneData[DatabaseInterface::TrackDataType::key_type::TitleRole] = newTrack.title();
+            } else {
+                const auto &fileUrl = newTrack.resourceURI();
+                oneData[DatabaseInterface::TrackDataType::key_type::TitleRole] = fileUrl.fileName();
+            }
+
+            oneData[DatabaseInterface::TrackDataType::key_type::ArtistRole] = newTrack.artist();
+            oneData[DatabaseInterface::TrackDataType::key_type::AlbumRole] = newTrack.albumName();
+            oneData[DatabaseInterface::TrackDataType::key_type::AlbumIdRole] = newTrack.albumId();
+            oneData[DatabaseInterface::TrackDataType::key_type::TrackNumberRole] = newTrack.trackNumber();
+            oneData[DatabaseInterface::TrackDataType::key_type::DiscNumberRole] = newTrack.discNumber();
+            oneData[DatabaseInterface::TrackDataType::key_type::DurationRole] = newTrack.duration();
+            oneData[DatabaseInterface::TrackDataType::key_type::MilliSecondsDurationRole] = newTrack.duration().msecsSinceStartOfDay();
+            oneData[DatabaseInterface::TrackDataType::key_type::ResourceRole] = newTrack.resourceURI();
+            oneData[DatabaseInterface::TrackDataType::key_type::ImageUrlRole] = newTrack.albumCover();
+
+            Q_EMIT trackHasChanged(oneData);
 
             return;
         }
@@ -156,29 +170,54 @@ void TracksListener::trackByFileNameInList(const QUrl &fileName)
 
     d->mTracksByIdSet.insert(newTrackId);
 
-    auto newTrack = d->mDatabase->trackFromDatabaseId(newTrackId);
+    auto newTrack = d->mDatabase->trackDataFromDatabaseId(newTrackId);
 
-    qDebug() << "TracksListener::trackByFileNameInList" << newTrackId << newTrack;
-
-    if (newTrack.isValid()) {
-        qDebug() << "TracksListener::trackByFileNameInList" << "trackHasChanged" << newTrack;
-        Q_EMIT trackHasChanged(newTrack);
+    if (!newTrack.isEmpty()) {
+        Q_EMIT trackHasChanged({newTrack});
     }
 }
 
-void TracksListener::trackByIdInList(qulonglong newTrackId)
+void TracksListener::newAlbumInList(qulonglong newDatabaseId, const QString &entryTitle)
 {
-    d->mTracksByIdSet.insert(newTrackId);
+    Q_EMIT tracksListAdded(newDatabaseId, entryTitle, ElisaUtils::Album, d->mDatabase->albumData(newDatabaseId));
+}
 
-    auto newTrack = d->mDatabase->trackFromDatabaseId(newTrackId);
-    if (newTrack.isValid()) {
-        Q_EMIT trackHasChanged(newTrack);
+void TracksListener::newEntryInList(qulonglong newDatabaseId,
+                                    const QString &entryTitle,
+                                    ElisaUtils::PlayListEntryType databaseIdType)
+{
+    switch (databaseIdType)
+    {
+    case ElisaUtils::Track:
+    {
+        d->mTracksByIdSet.insert(newDatabaseId);
+
+        auto newTrack = d->mDatabase->trackDataFromDatabaseId(newDatabaseId);
+        if (!newTrack.isEmpty()) {
+            Q_EMIT trackHasChanged(newTrack);
+        }
+        break;
+    }
+    case ElisaUtils::Artist:
+        newArtistInList(newDatabaseId, entryTitle);
+        break;
+    case ElisaUtils::FileName:
+        trackByFileNameInList(QUrl::fromLocalFile(entryTitle));
+        break;
+    case ElisaUtils::Album:
+        newAlbumInList(newDatabaseId, entryTitle);
+        break;
+    case ElisaUtils::Lyricist:
+    case ElisaUtils::Composer:
+    case ElisaUtils::Genre:
+    case ElisaUtils::Unknown:
+        break;
     }
 }
 
-void TracksListener::newArtistInList(const QString &artist)
+void TracksListener::newArtistInList(qulonglong newDatabaseId, const QString &artist)
 {
-    auto newTracks = d->mDatabase->tracksFromAuthor(artist);
+    auto newTracks = d->mDatabase->tracksDataFromAuthor(artist);
     if (newTracks.isEmpty()) {
         return;
     }
@@ -187,7 +226,7 @@ void TracksListener::newArtistInList(const QString &artist)
         d->mTracksByIdSet.insert(oneTrack.databaseId());
     }
 
-    Q_EMIT albumAdded(newTracks);
+    Q_EMIT tracksListAdded(newDatabaseId, artist, ElisaUtils::Artist, newTracks);
 }
 
 MusicAudioTrack TracksListener::scanOneFile(const QUrl &scanFile)
